function [b, u, P] = householder_vector(v)
	%HOUSEHOLDER_VECTOR Construct the Householder reflector for V. 
	%
	% [B, U] = HOUSEHOLDER_VECTOR(V) constructs a scalar B and a vector U such
	% that (I - B*U*U') * V is a multiple of e1 (the first column of the
	% identity matrix). 
	%
	% If the optional output parameter P is requested, the full matrix P
	% obtained by setting P = I - B*U*U' is built; note that this is fairly
	% inefficient in the general case, and operations with B, U should be
	% preferred. 

	alpha = norm(v);
	s = sign(v(1));

	u = v;
	% u(1) = v(1) + s * alpha;
	u(1) = -norm(v(2:end))^2 / (v(1) + s * alpha);

	b = 2 / (u' * u);

	if nargout > 2
		P = eye(size(v, 1)) - b * u * u';
	end

end

